# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: citylex.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='citylex.proto',
  package='citylex',
  syntax='proto2',
  serialized_pb=_b('\n\rcitylex.proto\x12\x07\x63itylex\"\x84\x04\n\x05\x45ntry\x12\x12\n\ncelex_freq\x18\x01 \x01(\x03\x12\x17\n\x0fsubtlex_uk_freq\x18\x02 \x01(\x03\x12\x15\n\rsubtlex_uk_cd\x18\x03 \x01(\x03\x12\x17\n\x0fsubtlex_us_freq\x18\x04 \x01(\x03\x12\x15\n\rsubtlex_us_cd\x18\x05 \x01(\x03\x12/\n\x0b\x63\x65lex_morph\x18\x06 \x03(\x0b\x32\x1a.citylex.Entry.MorphTriple\x12\x14\n\x0c\x65lp_morph_sp\x18\x07 \x01(\t\x12\x12\n\nelp_nmorph\x18\x08 \x01(\x05\x12\x34\n\x10udlexicons_morph\x18\t \x03(\x0b\x32\x1a.citylex.Entry.MorphTriple\x12\x32\n\x0eunimorph_morph\x18\n \x03(\x0b\x32\x1a.citylex.Entry.MorphTriple\x12\x12\n\ncelex_pron\x18\x0b \x03(\t\x12\x14\n\x0c\x63mudict_pron\x18\x0c \x03(\t\x12\x18\n\x10wikipron_uk_pron\x18\r \x03(\t\x12\x18\n\x10wikipron_us_pron\x18\x0e \x03(\t\x1a.\n\x0bMorphTriple\x12\r\n\x05lemma\x18\x01 \x02(\t\x12\x10\n\x08\x66\x65\x61tures\x18\x03 \x01(\t\x1a\x34\n\x0cSegmentation\x12\x14\n\x0csegmentation\x18\x01 \x02(\t\x12\x0e\n\x06nmorph\x18\x02 \x02(\x05\"s\n\x07Lexicon\x12*\n\x05\x65ntry\x18\x01 \x03(\x0b\x32\x1b.citylex.Lexicon.EntryEntry\x1a<\n\nEntryEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1d\n\x05value\x18\x02 \x01(\x0b\x32\x0e.citylex.Entry:\x02\x38\x01')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_ENTRY_MORPHTRIPLE = _descriptor.Descriptor(
  name='MorphTriple',
  full_name='citylex.Entry.MorphTriple',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='lemma', full_name='citylex.Entry.MorphTriple.lemma', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='features', full_name='citylex.Entry.MorphTriple.features', index=1,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=443,
  serialized_end=489,
)

_ENTRY_SEGMENTATION = _descriptor.Descriptor(
  name='Segmentation',
  full_name='citylex.Entry.Segmentation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='segmentation', full_name='citylex.Entry.Segmentation.segmentation', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='nmorph', full_name='citylex.Entry.Segmentation.nmorph', index=1,
      number=2, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=491,
  serialized_end=543,
)

_ENTRY = _descriptor.Descriptor(
  name='Entry',
  full_name='citylex.Entry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='celex_freq', full_name='citylex.Entry.celex_freq', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='subtlex_uk_freq', full_name='citylex.Entry.subtlex_uk_freq', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='subtlex_uk_cd', full_name='citylex.Entry.subtlex_uk_cd', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='subtlex_us_freq', full_name='citylex.Entry.subtlex_us_freq', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='subtlex_us_cd', full_name='citylex.Entry.subtlex_us_cd', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='celex_morph', full_name='citylex.Entry.celex_morph', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='elp_morph_sp', full_name='citylex.Entry.elp_morph_sp', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='elp_nmorph', full_name='citylex.Entry.elp_nmorph', index=7,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='udlexicons_morph', full_name='citylex.Entry.udlexicons_morph', index=8,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='unimorph_morph', full_name='citylex.Entry.unimorph_morph', index=9,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='celex_pron', full_name='citylex.Entry.celex_pron', index=10,
      number=11, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cmudict_pron', full_name='citylex.Entry.cmudict_pron', index=11,
      number=12, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='wikipron_uk_pron', full_name='citylex.Entry.wikipron_uk_pron', index=12,
      number=13, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='wikipron_us_pron', full_name='citylex.Entry.wikipron_us_pron', index=13,
      number=14, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_ENTRY_MORPHTRIPLE, _ENTRY_SEGMENTATION, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=27,
  serialized_end=543,
)


_LEXICON_ENTRYENTRY = _descriptor.Descriptor(
  name='EntryEntry',
  full_name='citylex.Lexicon.EntryEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='citylex.Lexicon.EntryEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='citylex.Lexicon.EntryEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=600,
  serialized_end=660,
)

_LEXICON = _descriptor.Descriptor(
  name='Lexicon',
  full_name='citylex.Lexicon',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='entry', full_name='citylex.Lexicon.entry', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_LEXICON_ENTRYENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=545,
  serialized_end=660,
)

_ENTRY_MORPHTRIPLE.containing_type = _ENTRY
_ENTRY_SEGMENTATION.containing_type = _ENTRY
_ENTRY.fields_by_name['celex_morph'].message_type = _ENTRY_MORPHTRIPLE
_ENTRY.fields_by_name['udlexicons_morph'].message_type = _ENTRY_MORPHTRIPLE
_ENTRY.fields_by_name['unimorph_morph'].message_type = _ENTRY_MORPHTRIPLE
_LEXICON_ENTRYENTRY.fields_by_name['value'].message_type = _ENTRY
_LEXICON_ENTRYENTRY.containing_type = _LEXICON
_LEXICON.fields_by_name['entry'].message_type = _LEXICON_ENTRYENTRY
DESCRIPTOR.message_types_by_name['Entry'] = _ENTRY
DESCRIPTOR.message_types_by_name['Lexicon'] = _LEXICON

Entry = _reflection.GeneratedProtocolMessageType('Entry', (_message.Message,), dict(

  MorphTriple = _reflection.GeneratedProtocolMessageType('MorphTriple', (_message.Message,), dict(
    DESCRIPTOR = _ENTRY_MORPHTRIPLE,
    __module__ = 'citylex_pb2'
    # @@protoc_insertion_point(class_scope:citylex.Entry.MorphTriple)
    ))
  ,

  Segmentation = _reflection.GeneratedProtocolMessageType('Segmentation', (_message.Message,), dict(
    DESCRIPTOR = _ENTRY_SEGMENTATION,
    __module__ = 'citylex_pb2'
    # @@protoc_insertion_point(class_scope:citylex.Entry.Segmentation)
    ))
  ,
  DESCRIPTOR = _ENTRY,
  __module__ = 'citylex_pb2'
  # @@protoc_insertion_point(class_scope:citylex.Entry)
  ))
_sym_db.RegisterMessage(Entry)
_sym_db.RegisterMessage(Entry.MorphTriple)
_sym_db.RegisterMessage(Entry.Segmentation)

Lexicon = _reflection.GeneratedProtocolMessageType('Lexicon', (_message.Message,), dict(

  EntryEntry = _reflection.GeneratedProtocolMessageType('EntryEntry', (_message.Message,), dict(
    DESCRIPTOR = _LEXICON_ENTRYENTRY,
    __module__ = 'citylex_pb2'
    # @@protoc_insertion_point(class_scope:citylex.Lexicon.EntryEntry)
    ))
  ,
  DESCRIPTOR = _LEXICON,
  __module__ = 'citylex_pb2'
  # @@protoc_insertion_point(class_scope:citylex.Lexicon)
  ))
_sym_db.RegisterMessage(Lexicon)
_sym_db.RegisterMessage(Lexicon.EntryEntry)


_LEXICON_ENTRYENTRY.has_options = True
_LEXICON_ENTRYENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
# @@protoc_insertion_point(module_scope)
